<?php
namespace MatthiasWeb\RealThumbnailGenerator\attachment;
use MatthiasWeb\RealThumbnailGenerator\base;
use MatthiasWeb\RealThumbnailGenerator\editor;

defined('ABSPATH') or die('No script kiddies please!'); // Avoid direct file request

/**
 * Regeneration process.
 */
class Regenerate extends base\Base {
    private static $me = null;

    /**
     * @see this::synchronize
     * @see this::synchronizeGroup
     * @see this::wp_die
     */
    private $wpDieSynchronize = [];

    private $synchronizeAction = '';

    /**
     * Hold the deletable files in the metadata so it can be deleted
     * from the the server.
     *
     * @param $data Array of updated attachment meta data
     * @param $post_id Attachment ID
     * @filter wp_update_attachment_metadata
     */
    public function wp_update_attachment_metadata($data, $post_id) {
        if (isset($GLOBALS['rtg_suppress_update_metadata']) && $GLOBALS['rtg_suppress_update_metadata'] === true) {
            return $data;
        }

        // Check if there are sizes which are deletable
        $check = Thumbnail::getInstance()->check($post_id, false);
        if (!is_wp_error($check) && count($check['deletable']) > 0) {
            // Try to merge with sizes
            if (!isset($data['sizes'])) {
                $data['sizes'] = [];
            }
            $data['sizes'] = array_unique(array_merge($data['sizes'], $check['deletable']), SORT_REGULAR);
        }

        // Correct the check from the actions
        if ($this->synchronizeAction === 'regenerate') {
            $check['mustGenerate'] = [];
        }

        // Save in own table for analyse purposes
        $this->synchronize($post_id, $check, defined('DOING_AJAX') && DOING_AJAX ? 'wp_die' : null);

        return $data;
    }

    /**
     * Synchronizes a check result with the rtg_attachments table so on this
     * base there can be made analysis.
     *
     * @param $postId The post ID
     * @param $check The result of Thumbnail::check()
     * @param $group If an array is given the the VALUES() SQL's are stored here. If you want to run
     *               the SQL you must use the this::synchronizeGroup function.
     * @return Boolean
     */
    public function synchronize($postId, $check, $group = null) {
        global $wpdb;

        $cnt = 0;
        $fileSize = 0;
        if (!is_wp_error($check)) {
            $cnt = count($check['deletable']);
            $fileSize = $check['deletableSpace'];
        }

        // Determine the group
        if (is_array($group)) {
            $doQuery = false;
        } elseif ($group !== 'wp_die') {
            $doQuery = true;
            $group = [];
        }

        // Prepare the query for the group
        $newValues = $wpdb->prepare(
            '(%d, %d, %d, %d)',
            $postId,
            $fileSize,
            $cnt,
            is_array($check) ? count($check['mustGenerate']) + count($check['notFound']) : 0
        );
        if ($group === 'wp_die') {
            $this->wpDieSynchronize[] = $newValues;
        } else {
            $group[] = $newValues;
            if ($doQuery) {
                $this->synchronizeGroup($group);
            }
        }
        return true;
    }

    /**
     * Synchronizes a group of attachments.
     *
     * @param $group An array of VALUES() SQL's
     * @see this::synchronize
     */
    public function synchronizeGroup($group) {
        global $wpdb;

        if (count($group) === 0) {
            return false;
        }

        $table_name = $this->getTableName('attachments');
        $valuesSql = implode(',', $group);
        $sql =
            "INSERT INTO $table_name (`post_id`, `free_space`, `cnt`, `cnt_regenerate`)
            VALUES " .
            $valuesSql .
            ' ON DUPLICATE KEY UPDATE free_space=VALUES(free_space), cnt=VALUES(cnt), cnt_regenerate=VALUES(cnt_regenerate)';

        $wpdb->query($sql);
        return true;
    }

    /**
     * Synchronizes a group of attachments at the end of the wordpress call.
     *
     * @see this::$wpDieSynchronize
     */
    public function wp_die($arg0) {
        $this->synchronizeGroup($this->wpDieSynchronize);
        return $arg0;
    }

    /**
     * Regenerates a set of Thumbnails. If you want to regenerate only a set of
     * defined sizes you have to define the $_REQUEST['onlyThisSizes'] = array('thumbnail')
     * variable.
     *
     * If you want to skip existing files define the $_REQUEST['skipExisting'] = true.
     *
     * @param $ids Array of IDs or single ID
     * @param $forceNew Determines if the new schema should be forced
     * @param $prepareForJs Use the Thumbnail::prepareForJS function to minify download
     * @return false or Array of post IDs with WP_Error or true
     */
    public function regenerate($ids, $forceNew = false, $prepareForJs = false) {
        if (is_numeric($ids)) {
            $ids = [$ids];
        }
        if (!is_array($ids)) {
            return false;
        }
        $result = [];

        // Iterate all ids
        foreach ($ids as $value) {
            try {
                set_error_handler([$this, 'set_error_handler']);
                $single = $this->single($value, $forceNew, $prepareForJs);

                if (is_wp_error($single)) {
                    // Throw
                    throw new \Exception($single->get_error_message());
                } else {
                    $result[$value] = $single;
                }
                restore_error_handler();
            } catch (\Exception $e) {
                $result[$value] = [
                    'error' => $e->getMessage()
                ];
                restore_error_handler();
            }
        }

        return $result;
    }

    /**
     * @see https://stackoverflow.com/a/1241751/5506547
     */
    public function set_error_handler($errno, $errstr, $errfile, $errline, $errcontext) {
        // error was suppressed with the @-operator
        if (0 === error_reporting()) {
            return false;
        }

        throw new \ErrorException($errstr, 0, $errno, $errfile, $errline);
    }

    /**
     * Regenerate a single post ID.
     *
     * @param $postId The post id
     * @param $forceNew Determines if the new schema should be forced
     * @param $prepareForJs Use the Thumbnail::prepareForJS function to minify download
     * @return true|WP_Error
     * @see this::wp_update_attachment_metadata
     */
    private function single($postId, $forceNew, $prepareForJs) {
        $check = Thumbnail::getInstance()->check($postId);
        if (is_wp_error($check)) {
            return $check;
        }

        // Do the regenerate
        $rtgEditor = editor\Editor::getInstance();
        $rtgEditor->activeRegenerationCheck = $check;
        $rtgEditor->activeRegenerationForceNew = $forceNew;

        $metadata = wp_generate_attachment_metadata($postId, $check['attachedFile']);

        $rtgEditor->activeRegenerationCheck = false;
        $rtgEditor->activeRegenerationCheck = false;

        if (is_array($metadata) && (isset($metadata['file']) || Thumbnail::getInstance()->isPdf($postId))) {
            $this->synchronizeAction = 'regenerate';
            $result = wp_update_attachment_metadata($postId, $metadata);
            $this->synchronizeAction = '';
            return $prepareForJs ? Thumbnail::getInstance()->prepareForJS($check) : $check;
        } else {
            return new \WP_Error(
                'generate-fails',
                sprintf(__('The thumbnails of image id %d could not be generated.', RTG_TD), $postId)
            );
        }
    }

    public static function getInstance() {
        if (self::$me == null) {
            self::$me = new Regenerate();
        }
        return self::$me;
    }
}
