<?php
namespace MatthiasWeb\RealThumbnailGenerator\editor;
use MatthiasWeb\RealThumbnailGenerator\base;
use MatthiasWeb\RealThumbnailGenerator\attachment;

defined('ABSPATH') or die('No script kiddies please!'); // Avoid direct file request

/**
 * Editor handler.
 */
class Editor extends base\Base {
    private static $me = null;

    /**
     * Provide two demo data to generate a test subfolder.
     * Here you can also see which variables are available within
     * the two percentage placeholders: %name% for example.
     */
    public $demo = [
        'size-identifier' => ['thumbnail', 'medium', 'large', 'custom', 'custom2'],
        'identifier-width' => ['150', '300', '1024', '300', '800'],
        'identifier-height' => ['150', '300', '1024', '1024', '940'],
        'name' => ['car', 'car', 'car', 'car', 'car'],
        'extension' => ['jpg', 'jpg', 'jpg', 'jpg', 'jpg'],
        'image-width' => ['130', '250', '1000', '250', '640'],
        'image-height' => ['145', '250', '1000', '640', '800']
    ];

    public $wrapping = [
        'size-identifier' => '<span>%size-identifier%</span>',
        'identifier-width' => '<span>%identifier-width%</span>',
        'identifier-height' => '<span>%identifier-height%</span>',
        'name' => '<span>%name%</span>',
        'extension' => '<span>%extension%</span>',
        'image-width' => '<span>%image-width%</span>',
        'image-height' => '<span>%image-height%</span>'
    ];

    public $unknownLabel = '_UNKNOWN_';

    /**
     * Save the active regenerating check to determine, if the post
     * already exist.
     *
     * @see attachment\Regenerate
     * @see GD Extended class
     * @see Imagick Extended class
     */
    public $activeRegenerationCheck = null;

    /**
     * Save the force new boolean if we want to regenerate a thumbnail.
     *
     * @see attachment\Regenerate
     * @see GD Extended class
     * @see Imagick Extended class
     */
    public $activeRegenerationForceNew = null;

    private $setDemo = false;

    /**
     * The used data for the replacement, see this::wrapping for example
     */
    public $data = [];

    private function __construct() {
        // Silence is golden.
    }

    /**
     * Add extended editors to the wordpress core
     * to handle the media library.
     */
    public function wp_image_editors($implementations) {
        $implns = RTG_NS . '\\editor\\impl\\';
        $replace = [
            'WP_Image_Editor_Imagick' => $implns . 'Imagick',
            'WP_Image_Editor_GD' => $implns . 'GD',

            // Custom editors with lazy loading
            'WP_Image_Editor_GD_Matted' => [
                'replace' => $implns . 'GD_Matted'
            ],
            'BFI_Image_Editor_Imagick_1_3' => [
                'replace' => $implns . 'BFIImagick_1_3'
            ],
            'BFI_Image_Editor_GD_1_3' => [
                'replace' => $implns . 'BFIGD_1_3'
            ],
            'WP_Thumb_Image_Editor_Imagick' => [
                'replace' => $implns . 'Thumb_Image_Editor_Imagick'
            ],
            'WP_Thumb_Image_Editor_GD' => [
                'replace' => $implns . 'Thumb_Image_Editor_GD'
            ],
            'EWWWIO_Imagick_Editor' => [
                'replace' => $implns . 'Ewww_Imagick'
            ],
            'EWWWIO_GD_Editor' => [
                'replace' => $implns . 'Ewww_GD'
            ]
        ];

        // Replace implementations
        foreach ($replace as $key => $value) {
            if (($idx = array_search($key, $implementations)) !== false) {
                $implementations[$idx] = is_array($value) ? $value['replace'] : $value;
            }
        }
        return array_values($implementations);
    }

    /**
     * Validate the thumbnail configuration.
     *
     * @param $patternSubfolder The string with placeholders %size-identifier%/%identifier-width%x%identifier-height%
     * @param $patternFilename The string with placeholders %name%-%size-identifier%.%extension%
     * @return Array
     */
    public function testThumbnailPath($patternSubfolder, $patternFilename) {
        $folders = [];
        $filenames = [];
        $pathes = [];
        $errors = [];
        $this->data = $this->demo;
        for ($i = 1; $i <= 5; $i++) {
            // Generate data
            $folder = $this->generate($patternSubfolder, $i);
            $filename = $this->generate($patternFilename, $i, 'replace_filename');

            // Catch errors
            if (!empty($filename) && empty($filename)) {
                $errors['filename_valid'] = __('The filename is not valid', RTG_TD);
            }
            if (!empty($filename) && strpos($filename, '.jpg') === false) {
                $errors['filename_ext'] = __('The filename has no extension', RTG_TD);
            }
            $join = path_join($folder, $filename);
            if (!$this->isValid($join)) {
                $errors['filename_ext'] = __('The pattern is invalid', RTG_TD);
            }

            $folders[] = $folder;
            $filenames[] = $filename;
            $pathes[] = $join;
        }

        // The filenames must be unique!
        if (count($pathes) !== count(array_unique($pathes))) {
            $errors['filename_unique'] = 'The filename pattern results not in an explicitness';
        }

        // Send result, is always successfully, but can contain errors
        return [
            'folders' => $folders,
            'filenames' => $filenames,
            'pathes' => $pathes,
            'errors' => array_unique($errors)
        ];
    }

    /**
     * Generate a subfolder/filename string from a given mode.
     *
     * The following placeholders are possible:
     *
     * $callback = "replace_subfolder"
     *  %size-identifier%: The name of the size ("medium", "full", "large", ...)
     *  %identifier-width%: The width of the identifier
     *  %identifier-height%: The height of the identifier
     *
     * $callback = "replace_filename"
     *  Same as above
     *  %name%: Name of the file
     *  %extension%: The extension of the file ("jpg", "png", ...)
     *  %image-width%: The width of the identifier
     *  %image-height%: The height of the identifier
     *
     * @param $string The string with placeholders
     * @param $mode Defines if it is the demo array or a custom array where we can replace
     * @param $callback The replace callback of this class
     * @return String
     * @see this::replace_subfolder
     * @see this::replace_filename
     */
    public function generate($string, $mode = false, $callback = 'replace_subfolder') {
        $this->setDemo($mode);
        $result = preg_replace_callback('/\%(.*)\%/U', [$this, $callback], $string);

        if (is_string($result)) {
            return trim($result, '/');
        } else {
            return '';
        }
    }

    /**
     * Wrap the placeholders with <span></span> tags.
     *
     * @see this::generate()
     */
    public function wrapSpan($string) {
        $this->data = $this->wrapping;
        return $this->generate($string);
    }

    /**
     * The main replacer for the subfolder.
     *
     * @see this::generate
     */
    public function replace_subfolder($match) {
        $id = trim(strtolower($match[1]));

        // Demo data
        if (!is_object($this->setDemo) && isset($this->data[$id])) {
            $useData = $this->data[$id];
            if (is_array($useData)) {
                return $useData[$this->setDemo - 1];
            } else {
                return $useData;
            }
        }

        return $this->unknownLabel;
    }

    /**
     * The main replacer for the filename.
     *
     * @see this::generate
     */
    public function replace_filename($match) {
        // Use variables from subfolder
        $fromSubfolder = $this->replace_subfolder($match);
        if ($fromSubfolder != $this->unknownLabel) {
            return $fromSubfolder;
        }

        // Use variables from the filename
        $match = trim(strtolower($match[1]));
        if (!is_object($this->setDemo) && isset($this->data[$match])) {
            $useData = $this->data[$match];
            if (is_array($useData)) {
                return $useData[$this->setDemo - 1];
            } else {
                return $useData;
            }
        }
        return $this->unknownLabel;
    }

    public function isValid($path) {
        return strpos($path, $this->unknownLabel) === false;
    }

    public function setDemo($demo = null) {
        $old = $this->setDemo;
        if ($demo !== null) {
            $this->setDemo = $demo;
        }
        return $old;
    }

    public static function getInstance() {
        if (self::$me == null) {
            self::$me = new Editor();
        }
        return self::$me;
    }
}

?>
