<?php
namespace MatthiasWeb\RealThumbnailGenerator\general;
use MatthiasWeb\RealThumbnailGenerator\base;
use MatthiasWeb\RealThumbnailGenerator\rest;

defined('ABSPATH') or die('No script kiddies please!'); // Avoid direct file request

/**
 * Asset management for frontend scripts and styles.
 */
class Assets extends base\Assets {
    /**
     * Enqueue scripts and styles depending on the type. This function is called
     * from both admin_enqueue_scripts and wp_enqueue_scripts. You can check the
     * type through the $type parameter. In this function you can include your
     * external libraries from public/lib, too.
     *
     * @param string $type The type (see base\Assets constants)
     */
    public function enqueue_scripts_and_styles($type) {
        $publicFolder = $this->getPublicFolder();
        $isDebug = $this->isScriptDebug();
        $dpSuffix = $isDebug ? 'development' : 'production.min';
        $minSuffix = $isDebug ? '' : '.min';

        if ($type === base\Assets::TYPE_ADMIN) {
            // ES6/ES7 polyfill
            $this->enqueueLibraryScript('es6-shim', 'es6-shim/es6-shim.min.js');
            $this->enqueueLibraryScript('es7-shim', 'es7-shim/dist/es7-shim.min.js', ['es6-shim']);

            // React (first check version is minium 16.8 so hooks work correctly)
            $coreReact = wp_scripts()->query('react');
            if ($coreReact !== false && version_compare($coreReact->ver, '16.8', '<')) {
                wp_deregister_script('react');
                wp_deregister_script('react-dom');
            }
            $this->enqueueLibraryScript('react', 'react/umd/react.' . $dpSuffix . '.js');
            $this->enqueueLibraryScript('react-dom', 'react-dom/umd/react-dom.' . $dpSuffix . '.js', 'react');

            // mobx
            $this->enqueueLibraryScript('mobx', 'mobx/lib/mobx.umd' . $minSuffix . '.js');

            // mobx-state-tree
            $this->enqueueLibraryScript('mobx-state-tree', 'mobx-state-tree/dist/mobx-state-tree.umd.js', ['mobx']);

            // react-window
            $this->enqueueLibraryScript(
                'react-window',
                'react-window/dist/index-' . ($isDebug ? 'dev' : 'prod') . '.umd.js',
                ['react', 'react-dom']
            );

            // Your assets implementation here... See base\Assets for enqueue* methods
            $scriptDeps = ['react-dom', 'moment', 'wp-i18n'];
            $handle = RTG_SLUG . '-admin';
            $this->enqueueScript($handle, 'admin.' . ($this->isPro() ? 'pro' : 'lite') . '.js', $scriptDeps);
            $this->enqueueStyle($handle, 'admin.css');
            wp_set_script_translations($handle, RTG_TD, path_join(RTG_PATH, base\Assets::PUBLIC_JSON_I18N));
            wp_localize_script($handle, RTG_OPT_PREFIX . 'Opts', $this->localizeScript($type));
        }

        // Localize with a window.process.env variable for libraries relying on it (MST for example)
        wp_localize_script('react', 'process', [
            'env' => ['NODE_ENV' => $isDebug ? 'development' : 'production']
        ]);
    }

    /**
     * Localize the WordPress backend and frontend. If you want to provide URLs to the
     * frontend you have to consider that some JS libraries do not support umlauts
     * in their URI builder. For this you can use base\Assets#getAsciiUrl.
     *
     * @return array
     */
    public function localizeScript($context) {
        require_once ABSPATH . WPINC . '/class-wp-image-editor.php';
        require_once ABSPATH . WPINC . '/class-wp-image-editor-gd.php';
        require_once ABSPATH . WPINC . '/class-wp-image-editor-imagick.php';

        $core = Core::getInstance();
        $isLicenseActivated = $this->isPro() ? $core->getUpdater()->isActivated() : true;
        $isLicenseNoticeDismissed = $core->isLicenseNoticeDismissed();

        $common = [
            'slug' => RTG_SLUG,
            'textDomain' => RTG_TD,
            'version' => RTG_VERSION
        ];

        // We put custom variables to "others" because if you put for example
        // a boolean to the first-level it is interpreted as "1" instead of true.
        if ($context === base\Assets::TYPE_ADMIN) {
            return array_merge($common, [
                'others' => [
                    'isPro' => $this->isPro(),
                    'showProHints' => current_user_can('install_plugins'),
                    'canManageOptions' => current_user_can('manage_options'),
                    'proUrl' => RTG_PRO_VERSION,
                    'thumbnailFolder' => get_option(RTG_OPT_PREFIX . '_thumbnail_folder', ''),
                    'thumbnailFilename' => get_option(RTG_OPT_PREFIX . '_thumbnail_filename', ''),
                    'chunkSize' => get_option(RTG_OPT_PREFIX . '_chunk_size', 8),
                    'pluginsUrl' => admin_url('plugins.php'),
                    'showLicenseNotice' =>
                        !$isLicenseActivated && !$isLicenseNoticeDismissed && current_user_can('install_plugins'),
                    'implementations' => apply_filters('wp_image_editors', [
                        'WP_Image_Editor_Imagick',
                        'WP_Image_Editor_GD'
                    ])
                ],
                'restUrl' => $this->getAsciiUrl(rest\Service::getUrl(rest\Service::SERVICE_NAMESPACE)),
                'restRoot' => $this->getAsciiUrl(get_rest_url()),
                'restQuery' => ['_v' => RTG_VERSION],
                'restNonce' => wp_installing() && !is_multisite() ? '' : wp_create_nonce('wp_rest'),
                'publicUrl' => trailingslashit(plugins_url('public', RTG_FILE))
            ]);
        }
    }

    /**
     * Enqueue scripts and styles for admin pages.
     */
    public function admin_enqueue_scripts() {
        $this->enqueue_scripts_and_styles(base\Assets::TYPE_ADMIN);
    }

    /**
     * Enqueue scripts and styles for frontend pages.
     */
    public function wp_enqueue_scripts() {
        $this->enqueue_scripts_and_styles(base\Assets::TYPE_FRONTEND);
    }
}
