<?php
namespace MatthiasWeb\RealThumbnailGenerator\rest;
use MatthiasWeb\RealThumbnailGenerator\base;
use MatthiasWeb\RealThumbnailGenerator\general;
use MatthiasWeb\RealThumbnailGenerator\attachment;

defined('ABSPATH') or die('No script kiddies please!'); // Avoid direct file request

/**
 * Create a REST Service for the analyse / bulk regenerate process.
 */
class Analyse extends base\Base {
    /**
     * Register endpoints.
     */
    public function rest_api_init() {
        register_rest_route(Service::SERVICE_NAMESPACE, '/stats', [
            'methods' => 'GET',
            'callback' => [$this, 'routeStats']
        ]);

        register_rest_route(Service::SERVICE_NAMESPACE, '/attachments', [
            'methods' => 'GET',
            'callback' => [$this, 'routeAttachments']
        ]);

        register_rest_route(Service::SERVICE_NAMESPACE, '/attachments/(?P<id>\d+)', [
            'methods' => 'GET',
            'callback' => [$this, 'routeSingle']
        ]);

        register_rest_route(Service::SERVICE_NAMESPACE, '/attachments', [
            'methods' => 'POST',
            'callback' => [$this, 'routeRegenerate']
        ]);

        register_rest_route(Service::SERVICE_NAMESPACE, '/attachments/(?P<id>\d+)', [
            'methods' => 'POST',
            'callback' => [$this, 'routeRegenerateSingle']
        ]);

        register_rest_route(Service::SERVICE_NAMESPACE, '/attachments/(?P<id>\d+)', [
            'methods' => 'DELETE',
            'callback' => [$this, 'routeDeleteSingle']
        ]);

        register_rest_route(Service::SERVICE_NAMESPACE, '/attachments', [
            'methods' => 'DELETE',
            'callback' => [$this, 'routeDelete']
        ]);
    }

    /**
     * @api {get} /realthumbnailgenerator/v1/stats Get stats
     * @apiHeader {string} X-WP-Nonce
     * @apiName GetStats
     * @apiGroup Analyse
     * @apiVersion 0.1.0
     */
    public function routeStats($request) {
        if (($permit = Service::permit('upload_files')) !== null) {
            return $permit;
        }
        return new \WP_REST_Response(general\Analyse::getInstance()->fetchStats());
    }

    /**
     * @api {get} /realthumbnailgenerator/v1/attachments Get attachments
     * @apiHeader {string} X-WP-Nonce
     * @apiParam {number} [posts_per_page=40]
     * @apiParam {number} [page=1]
     * @apiName GetAttachments
     * @apiGroup Analyse
     * @apiVersion 0.1.0
     */
    public function routeAttachments($request) {
        if (($permit = Service::permit('upload_files')) !== null) {
            return $permit;
        }
        return new \WP_REST_Response(
            general\Analyse::getInstance()->fetchAttachments(
                $request->get_param('posts_per_page'),
                $request->get_param('page')
            )
        );
    }

    /**
     * @api {get} /realthumbnailgenerator/v1/attachments/:id Get single attachment
     * @apiHeader {string} X-WP-Nonce
     * @apiParam {number} [posts_per_page=40]
     * @apiParam {number} [page=1]
     * @apiName GetSingle
     * @apiGroup Analyse
     * @apiVersion 0.1.0
     */
    public function routeSingle($request) {
        if (($permit = Service::permit('upload_files')) !== null) {
            return $permit;
        }
        $id = $request->get_param('id');
        $check = $this->prepareSingleCheck($id);
        if (is_wp_error($check)) {
            return $check;
        }
        return new \WP_REST_Response($check);
    }

    private function prepareSingleCheck($id) {
        $check = attachment\Thumbnail::getInstance()->check($id);
        if (is_wp_error($check)) {
            return $check;
        }
        $check['usedSchema'] = path_join($check['schemaf'], $check['schema']);
        $check['newSchema'] = attachment\Thumbnail::getInstance()->getSchema();
        $check['deletableSpaceFormat'] = size_format($check['deletableSpace']);
        $check['filesizeSumFormat'] = size_format($check['filesizeSum']);
        unset($check['meta']);
        unset($check['attachedFile']);
        unset($check['unused']);
        unset($check['mustGenerate']);
        unset($check['notFound']);
        unset($check['deletable']);
        unset($check['available']);
        return $check;
    }

    /**
     * @api {post} /realthumbnailgenerator/v1/attachments Regenerate attachments
     * @apiHeader {string} X-WP-Nonce
     * @apiParam {number} [posts_per_page=40]
     * @apiParam {number} [page=1]
     * @apiParam {string[]} sizes The sizes to regenerate
     * @apiParam {boolean} [forceNewSchema] If true all sizes are regenerated with the new schema
     * @apiParam {boolean} [skipExisting] If true existing files are skipped
     * @apiName RegenerateAttachments
     * @apiGroup Analyse
     * @apiVersion 0.1.0
     */
    public function routeRegenerate($request) {
        if (($permit = Service::permit('upload_files')) !== null) {
            return $permit;
        }

        $posts_per_page = $request->get_param('posts_per_page');
        $page = $request->get_param('page');
        $attachments = general\Analyse::getInstance()->fetchAttachments($posts_per_page, $page);
        $sizes = $request->get_param('sizes');
        $forceNewSchema = $request->get_param('forceNewSchema') === 'true';
        $skipExisting = $request->get_param('skipExisting') === 'true';
        $ids = [];
        foreach ($attachments as $check) {
            $ids[] = $check['id'];
        }

        // Register only sizes in global variable
        if (is_array($sizes) && count($sizes) > 0) {
            $_REQUEST['onlyThisSizes'] = $sizes;
        }
        if ($skipExisting) {
            $_REQUEST['skipExisting'] = true;
        }

        // Do the regeneration
        include_once path_join(ABSPATH, 'wp-admin/includes/image.php'); // Not included in REST
        $result = attachment\Regenerate::getInstance()->regenerate($ids, $forceNewSchema, true);
        $fetch = general\Analyse::getInstance()->fetchAttachments($posts_per_page, $page);

        // Merge errors
        foreach ($fetch as $key => $check) {
            if (isset($result[$check['id']]) && isset($result[$check['id']]['error'])) {
                $fetch[$key]['error'] = $result[$check['id']]['error'];
            }
        }

        return new \WP_REST_Response($fetch);
    }

    /**
     * @api {post} /realthumbnailgenerator/v1/attachments/:id Regenerate single attachment
     * @apiHeader {string} X-WP-Nonce
     * @apiParam {boolean} [forceNewSchema] If true all sizes are regenerated with the new schema
     * @apiName RegenerateAttachment
     * @apiGroup Analyse
     * @apiVersion 0.1.0
     */
    public function routeRegenerateSingle($request) {
        if (($permit = Service::permit('upload_files')) !== null) {
            return $permit;
        }

        $id = $request->get_param('id');
        $forceNewSchema = $request->get_param('forceNewSchema') === 'true';

        // Do the regeneration
        include_once path_join(ABSPATH, 'wp-admin/includes/image.php'); // Not included in REST
        $result = attachment\Regenerate::getInstance()->regenerate([$id], $forceNewSchema, true);

        // Output
        $check = $this->prepareSingleCheck($id);
        if (is_wp_error($check)) {
            return $check;
        }

        // Merge error
        if (isset($result[$id]['error'])) {
            $check['error'] = $result[$id]['error'];
        }

        return new \WP_REST_Response($check);
    }

    /**
     * @api {delete} /realthumbnailgenerator/v1/attachments/:id Delete unused sizes
     * @apiHeader {string} X-WP-Nonce
     * @apiName DeleteUnusedAttachment
     * @apiGroup Analyse
     * @apiVersion 0.1.0
     */
    public function routeDeleteSingle($request) {
        if (($permit = Service::permit('upload_files')) !== null) {
            return $permit;
        }

        $id = $request->get_param('id');

        // Do the regeneration
        include_once path_join(ABSPATH, 'wp-admin/includes/image.php'); // Not included in REST
        $result = attachment\Thumbnail::getInstance()->deleteUnused([$id]);
        if (is_wp_error($result)) {
            return $result;
        }

        // Output
        $check = $this->prepareSingleCheck($id);
        if (is_wp_error($check)) {
            return $check;
        }
        return new \WP_REST_Response($check);
    }

    /**
     * @api {delete} /realthumbnailgenerator/v1/attachments Delete all unused sizes
     * @apiHeader {string} X-WP-Nonce
     * @apiParam {number} [posts_per_page=40]
     * @apiParam {number} [page=1]
     * @apiName DeleteAllUnused
     * @apiGroup Analyse
     * @apiVersion 0.1.0
     */
    public function routeDelete($request) {
        if (($permit = Service::permit('upload_files')) !== null) {
            return $permit;
        }

        $posts_per_page = $request->get_param('posts_per_page');
        $page = $request->get_param('page');
        $attachments = general\Analyse::getInstance()->fetchAttachments($posts_per_page, $page);
        $ids = [];
        foreach ($attachments as $check) {
            $ids[] = $check['id'];
        }

        // Do the regeneration
        include_once path_join(ABSPATH, 'wp-admin/includes/image.php'); // Not included in REST

        $result = attachment\Thumbnail::getInstance()->deleteUnused($ids);
        if (is_wp_error($result)) {
            return $result;
        }

        return new \WP_REST_Response(general\Analyse::getInstance()->fetchAttachments($posts_per_page, $page));
    }
}
