<?php
namespace MatthiasWeb\RealThumbnailGenerator\rest;
use MatthiasWeb\RealThumbnailGenerator\base;
use MatthiasWeb\RealThumbnailGenerator\general;
use MatthiasWeb\RealThumbnailGenerator\editor;

defined('ABSPATH') or die('No script kiddies please!'); // Avoid direct file request

/**
 * Create a REST Service.
 */
class Service extends base\Base {
    /**
     * The namespace for this service.
     *
     * @see getUrl()
     */
    const SERVICE_NAMESPACE = 'realthumbnailgenerator/v1';

    /**
     * Register endpoints.
     */
    public function rest_api_init() {
        register_rest_route(Service::SERVICE_NAMESPACE, '/plugin', [
            'methods' => 'GET',
            'callback' => [$this, 'routePlugin']
        ]);

        register_rest_route(Service::SERVICE_NAMESPACE, '/settings', [
            'methods' => 'PUT',
            'callback' => [$this, 'routePutSettings']
        ]);

        register_rest_route(Service::SERVICE_NAMESPACE, '/settings/verify', [
            'methods' => 'POST',
            'callback' => [$this, 'routeVerifySettings']
        ]);

        register_rest_route(Service::SERVICE_NAMESPACE, '/notice/license', [
            'methods' => 'DELETE',
            'callback' => [$this, 'routeNoticeDismissLicense']
        ]);
    }

    /**
     * @api {get} /realthumbnailgenerator/v1/plugin Get plugin information
     * @apiHeader {string} X-WP-Nonce
     * @apiName GetPlugin
     * @apiGroup Plugin
     *
     * @apiSuccessExample {json} Success-Response:
     * {
     *     WC requires at least: "",
     *     WC tested up to: "",
     *     Name: "WP ReactJS Starter",
     *     PluginURI: "https://matthias-web.com/wordpress",
     *     Version: "0.1.0",
     *     Description: "This WordPress plugin demonstrates how to setup a plugin that uses React and ES6 in a WordPress plugin. <cite>By <a href="https://matthias-web.com">Matthias Guenter</a>.</cite>",
     *     Author: "<a href="https://matthias-web.com">Matthias Guenter</a>",
     *     AuthorURI: "https://matthias-web.com",
     *     TextDomain: "wp-reactjs-starter",
     *     DomainPath: "/languages",
     *     Network: false,
     *     Title: "<a href="https://matthias-web.com/wordpress">WP ReactJS Starter</a>",
     *     AuthorName: "Matthias Guenter"
     * }
     * @apiVersion 0.1.0
     */
    public function routePlugin() {
        return new \WP_REST_Response(general\Core::getInstance()->getPluginData());
    }

    /**
     * @api {get} /realthumbnailgenerator/v1/settings Update settings
     * @apiHeader {string} X-WP-Nonce
     * @apiParam {string} thumbnailFolder The thumbnail folder scheme
     * @apiParam {string} thumbnailFilename The thumbnail filename scheme
     * @apiParam {number} chunkSize The chunk size
     * @apiName UpdateSettings
     * @apiGroup Settings
     * @apiVersion 0.1.0
     */
    public function routePutSettings($request) {
        if (($permit = Service::permit('manage_options')) !== null) {
            return $permit;
        }

        $thumbnailFolder = $request->get_param('thumbnailFolder');
        $thumbnailFilename = $request->get_param('thumbnailFilename');
        $chunkSize = $request->get_param('chunkSize');
        update_option(RTG_OPT_PREFIX . '_thumbnail_folder', $thumbnailFolder);
        update_option(RTG_OPT_PREFIX . '_thumbnail_filename', $thumbnailFilename);
        update_option(RTG_OPT_PREFIX . '_chunk_size', $chunkSize);

        return new \WP_REST_Response(true);
    }

    /**
     * @api {post} /realthumbnailgenerator/v1/settings/verify Verify settings
     * @apiHeader {string} X-WP-Nonce
     * @apiParam {string} thumbnailFolder The thumbnail folder scheme
     * @apiParam {string} thumbnailFilename The thumbnail filename scheme
     * @apiName VerifySettings
     * @apiGroup Settings
     * @apiVersion 0.1.0
     */
    public function routeVerifySettings($request) {
        if (($permit = Service::permit('manage_options')) !== null) {
            return $permit;
        }

        $thumbnailFolder = $request->get_param('thumbnailFolder');
        $thumbnailFilename = $request->get_param('thumbnailFilename');
        return new \WP_REST_Response(
            editor\Editor::getInstance()->testThumbnailPath($thumbnailFolder, $thumbnailFilename)
        );
    }

    /**
     * @api {delete} /realthumbnailgenerator/v1/notice/license Dismiss the license notice for a given time (transient)
     * @apiName DismissLicenseNotice
     * @apiGroup Plugin
     * @apiVersion 0.1.0
     * @apiPermission install_plugins
     */
    public function routeNoticeDismissLicense() {
        if (($permit = Service::permit('install_plugins')) !== null) {
            return $permit;
        }
        general\Core::getInstance()->isLicenseNoticeDismissed(true);
        return new \WP_REST_Response(true);
    }

    /**
     * Checks if the current user has a given capability and throws an error if not.
     *
     * @param string $cap The capability
     * @throws \WP_Error
     */
    public static function permit($cap = 'upload_files') {
        if (!current_user_can($cap)) {
            return new \WP_Error('rest_rtg_forbidden', __('Forbidden'), array('status' => 403));
        }
        return null;
    }

    /**
     * Get the wp-json URL for a defined REST service.
     *
     * @param string $namespace The prefix for REST service
     * @param string $endpoint The path appended to the prefix
     * @return String Example: https://example.com/wp-json
     * @example Service::url(Service::SERVICE_NAMESPACE) // => main path
     */
    public static function getUrl($namespace, $endpoint = '') {
        return site_url(rest_get_url_prefix()) . '/' . $namespace . '/' . $endpoint;
    }
}
